%given edge endpoints 'stuff' and number of boundary nodes 'n', set up ordering of 
%gammas and lambdas and compute all possible ranks of the differential
%
%as pivots are produced in differential, check for possible zeros and
%branch into 2 processes, one of which assumes the zero condition and one
%that doesn't, keeping track of assumptions and conditions
%
%K is Kirchhoff matrix
%L is response
%D is differential of map from conductances to response matrix entries
%R contains rref forms of D possibly under multiple conditions
%pivots are the pivots in R
%C contains the zero conditions assumed
%A contains the non-zero assumptions used
%stuff can be Kirchhoff matrix or list of edge indices given as a num_edges by 2 matrix
%
%usage: function [K,L,D,R,pivots,ranks,C,A] = getranks(stuff,n)
function [K,L,D,R,pivots,ranks,C,A] = getranks(stuff,n) 
[M,N] = size(stuff);
if (M==N & M>2) %assumes at least 3 nodes if given a Kirchhoff matrix
    K = stuff; %use K to define g
    count = 1;
    for i = 1:M-1
        for j = i+1:N
            if (K(i,j) ~= 0)
                g(count,1) = i;
                g(count,2) = j;
                count = count + 1;
            end
        end
    end   
else
    g = stuff; %if stuff already defined g
end
num_edges = size(g,1);
biggest_index = max(max(g));
num_entries = n*(n-1)/2;

%make Kirchhoff matrix (symbolically)
K = sym(zeros(biggest_index)); %vertices numbered from 1 to biggest_index with bndry first
for i = 1:num_edges
    eval(['syms g' num2str(i) ' real']);
    K(g(i,1),g(i,2)) = eval(['-g' num2str(i)]);    
end
K = K + K';
S = -sum(K);
for i=1:biggest_index
    K(i,i) = S(i);
end
K = sym(K);

%get response matrix
L = sym(getL(K,n));

%compute differential D, a num_entries by num_edges matrix
D = sym(zeros(num_entries,num_edges));
row = 1;
for i=1:n-1
    for j=i+1:n       
        for k = 1:num_edges  %computes derivatives using MATLAB's 'diff' command
            D(row,k) = eval(['simplify(diff(L(i,j),g' num2str(k) '))']);
        end
        row = row + 1;
    end
end

%put in (almost) rref form leaving pivots as symbolic expressions
R(:,:,1) = D;
assumptions = sym(-1); %expressions assumed to be nonzero (without being sure)
UA = 0;  %describes where assumptions are used
conditions = sym(-1); %list of expressions that can sometimes be zero
UC = 0; %describes which cases use these zero conditions
startcolumn = 1;
caseindex = 1;
while (caseindex <= size(R,3)) %number of cases can possibly increase 
    Rtemp = R(:,:,caseindex);
    for j = startcolumn(caseindex):num_edges %loop over columns
        %permute rows and columns to get nonzero pivot
        k = j;
        while (k<=num_entries & Rtemp(j,j) == 0 ) 
            l = j;            
            while (l<num_edges & Rtemp(j,j) == 0 ) 
                temp = Rtemp(:,j);
                Rtemp(:,j) = Rtemp(:,l+1); %permuting columns
                Rtemp(:,l+1) = temp;
                l = l + 1;
            end
            if (k==num_entries | Rtemp(j,j) ~= 0 ) 
                break;
            end
            temp = Rtemp(j,:);
            Rtemp(j,:) = Rtemp(k+1,:); %permuting rows
            Rtemp(k+1,:) = temp;
            k = k + 1;
        end
        
        %break if no more nonzero pivots are to be found
        if ( (j <= num_entries & Rtemp(j,j) == 0) | (j > num_entries) )    
            break;
        end  
        [b,v] = cbz(Rtemp(j,j));
        
        if (b==0 | b==-1)    
            %perform gaussian elimination
            for i=j+1:num_entries  
                m = simplify(Rtemp(i,j)/Rtemp(j,j));
                Rtemp(i,:) = simplify(Rtemp(i,:) - Rtemp(j,:)*m);        
            end
            
            if (b==-1) %add assumptions
                if (assumptions(1) == -1)
                    len = 1;
                    assumptions(len) = Rtemp(j,j);
                else                  
                    assumptions(length(assumptions)+1) = Rtemp(j,j);
                end
                UA(caseindex,length(assumptions)) = 1;
            end
            
        elseif (b == 1)
            %expansion case ie: adding zero conditions
            if (conditions(1) == -1)
                len = 1;
                conditions(len) = Rtemp(j,j);
            else              
                conditions(length(conditions)+1) = Rtemp(j,j);
            end           
            R(:,:,size(R,3)+1) = Rtemp;  
            UA(size(R,3),:) = UA(caseindex,:);
            UC(size(R,3),:) = UC(caseindex,:);
            UC(size(R,3),length(conditions)) = 1;
            startcolumn(size(R,3)) = j;
            if (length(v) == 4 & v == 'pfft')
                disp('something not right');
            end
            %substitute the v variable into the new row of R and simplify result
            R(:,:,size(R,3)) = simsub(R(:,:,size(R,3)),v,Rtemp(j,j));
        else
            disp('ERROR, no valid cases');
        end
        R(:,:,caseindex) = Rtemp;   
    end  
    numpivots = min(num_edges,num_entries);
    pivots(:,caseindex) = diag(R(1:numpivots,1:numpivots,caseindex));
    
    caseindex = caseindex + 1; %move on to next case
    
end

%need to produce [K,L,D,R,pivots,ranks,C,A]
for i = 1:size(pivots,2)
    if (isempty(min(find(pivots(:,i)==0))))
        ranks(i,1) = numpivots;
    else       
        ranks(i,1) = min(find(pivots(:,i)==0))-1;
    end
    if (isempty(find(UC(i,:)==1)))
        C(i,:) = sym(0);
    else
        C(i,1:length(conditions(find(UC(i,:)==1)))) = conditions(find(UC(i,:)==1));
    end
    if (isempty(find(UA(i,:)==1)))
        A(i,:) = sym(0);
    else
        A(i,1:length(assumptions(find(UA(i,:)==1)))) = assumptions(find(UA(i,:)==1));
    end    
end  

%still to do: 
%write program that uses this to examine K-star transformation one step at a times